#!/bin/bash
#

# build_mysql_client.sh <product_directory> <e5|e6> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e5|e6> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`
    qualver=`echo ${basequal} | sed -e 's/:/_/g'`
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual} -z ${product_dir}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual} -z ${product_dir}
    fi
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}
sourcetar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=mysql_client
origpkgver=v5_5_39
pkgver=${origpkgver}
ssibuildshims_version=v0_07_04
cmake_version=v3_0_1
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=mysql-${pkgdotver}

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${basequal}" = "e5" ] || [ "${basequal}" = "e4" ]
then
  cxxflg="-std=c++11 -Wno-deprecated-declarations"
elif [ "${basequal}" = "e6" ] 
then
  cxxflg="-std=c++1y -Wno-deprecated-declarations"
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || die "ERROR: fake declare failed"

setup ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir} || die "ERROR: fake setup failed."

# We need cmake
setup -B cmake ${cmake_version} || die "ERROR: setup -B cmake ${cmake_version} failed"

##CFLAGS="-O3" CXX=gcc CXXFLAGS="-O3 -felide-constructors \
 ##      -fno-exceptions -fno-rtti" ${sourcedir}/configure \
##       --prefix=${mydir} --without-server --with-mysqld-ldflags=-all-static
##

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

if [ ! -d ${sourcedir}/${pkgtarname} ]
then
   cd ${sourcedir}
   tar xf ${tardir}/${pkgtarname}.tar.gz
   patch -b -p0 < ${patchdir}/${pkgtarname}.patch
fi

cd ${blddir}

if [ ${basequal} = "e5" ] || [ "${basequal}" = "e4" ]
then
    CC=gcc CXX=g++ cmake -DCMAKE_INSTALL_PREFIX=${pkgdir} \
                   -DCMAKE_BUILD_TYPE=Relwithdebinfo \
		   -DCMAKE_CXX_FLAGS="${cxxflg}" \
		   -DWITHOUT_SERVER:BOOL=ON ${sourcedir}/${pkgtarname}
else
    CC=gcc CXX=g++ cmake -DCMAKE_INSTALL_PREFIX=${pkgdir} -DWITHOUT_SERVER:BOOL=ON ${sourcedir}/${pkgtarname}
fi
(( $? == 0 )) || die "ERROR: CMake failed."
make -j ${ncore} || die "ERROR: make failed."
make install || die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi
if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

declare_product

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "mysql_client is installed at ${MYSQL_CLIENT_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
